#include "pthr.h"

/* Init the pthr_shared */
void pthr_init(long nbthreads) {
  int status;
  /* Init the single wait */
  status = pthr_single_init(&(pthr_shared.singlewait), nbthreads);
  if (status != 0)
    PthrErrAbort(status,"pthr_single_init");
  /* Init the single last */
  status = pthr_singlelast_init(&(pthr_shared.singlelast), nbthreads);
  if (status != 0)
    PthrErrAbort(status,"pthr_singlelast_init");
  /* Init the mutex for reduction */
  status = pthread_mutex_init(&(pthr_shared.reduction), NULL);
  if (status != 0)
    PthrErrAbort(status,"pthread_mutex_init");
  /* Init private static data */
  status = pthread_key_create(&(pthr_shared.this), NULL);
  if (status != 0)
    PthrErrAbort(status,"pthread_key_create");
}

/* Compute the local bound for a loop
 * if the loop is for (i=a; i<b; i+=c)
 * it became
 * for (i=la; i<lb; i+=c)
 */
void pthr_distribbound(int nbrproc, int a, int b, int c, int * la, int * lb){
  pthr_private_t * myData;
  int nbrstep, numproc;
  /* Retrieve the number of thread */
  myData = (pthr_private_t *) pthread_getspecific(pthr_shared.this);
  numproc = myData->me;
  /* Compute local bound */
  nbrstep = (1 + (b-a-1)/c );
  *la = a + c* ((int) (numproc * ( (double) nbrstep )/ nbrproc));
  *lb = a+1-c + c* ((int) ( (numproc+1) * ( (double) nbrstep )/ nbrproc));
  /*printf("bound %d %d %d %d %d %d %d\n",
    nbrproc, numproc, a,b,c, *la, *lb);*/}

void pthr_destroy() {
  int status;

  status = pthr_single_destroy(&(pthr_shared.singlewait) );
  if (status != 0)
    PthrErrAbort(status,"pthr_single_destroy");
  /* Init the single last */
  status = pthr_singlelast_destroy(&(pthr_shared.singlelast));
  if (status != 0)
    PthrErrAbort(status,"pthr_singlelast_destroy");
  /* Init the mutex for reduction */
  status = pthread_mutex_destroy(&(pthr_shared.reduction));
  if (status != 0)
    PthrErrAbort(status,"pthread_mutex_destroy");
  /* Init private static data */
  status = pthread_key_delete(pthr_shared.this);
  if (status != 0)
    PthrErrAbort(status,"pthread_key_delete");
}
