#ifndef PTHR_H
#define PTHR_H

/* Tool for pthread version */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>
#include "pthrsingle.h"
#include "pthrsinglelast.h"

/* abort on error in pthread library with some info */
#define PthrErrAbort(status,text) do {					\
    fprintf(stderr, "%s at %s:%d %s\n",					\
	    (text), __FILE__, __LINE__, strerror(status)); abort(); }	\
  while(0)

/* Check Malloc error */
#define CheckMalloc(ptr) do {				\
    if ( (ptr) == NULL) {				\
      fprintf(stderr,"Not enought memory at %s:%d\n",	\
	      __FILE__, __LINE__); abort();}		\
  }							\
  while(0)

/* Shared data for threads synchro */
typedef struct _pthr_shared_t {
  /* Single wait structure */
  pthr_single_t singlewait;
  /* Single last structure */
  pthr_singlelast_t singlelast;
  /* Reduction mutex */
  pthread_mutex_t reduction;
  /* Key for access private part */
  pthread_key_t this;
} pthr_shared_t;

/* Private data */
typedef struct _pthr_private_t {
  /* Rank in pthreads pool
     usefull to compute local part of for loop in distribbound */
  int me;
} pthr_private_t;

/* Static shared object */
pthr_shared_t pthr_shared;

/* Init the pthr_shared */
void pthr_init(long nbthreads);
/* Compute the local bound for a loop
 * if the loop is
 * for (i=imin; i<imax; i+=istep)
 * it became
 * for (i=*ilmin; i<*ilmax; i+=istep)
 */
void pthr_distribbound(int nbrthr, int imin, int imax, int istep,
		       int * ilmint, int * ilmax);

#endif
