#include "pthrsingle.h"
#include <errno.h>
/* Init the single structure with the numbers of threads involved */
int pthr_single_init(pthr_single_t *single, int count)
{
  int code;
  /* Init mutex & cond struct */
  code = pthread_mutex_init( &(single->mutex), NULL);
  if (code != 0)
    return code;
  code = pthread_cond_init( &(single->cond), NULL);
  if (code != 0) {
    pthread_mutex_destroy( &(single->mutex) );
    return code; }
  single->valid = PTHR_SINGLE_VALID; /* Now we can use single struct */
  single->threshold = count;
  single->counter = count;
  single->first = 1;
  single->cycle = 0;
  return 0;
}
/* Destroy the single structure */
int pthr_single_destroy(pthr_single_t *single)
{
  int code,code2;
  /* Is single valid */
  if (single->valid != PTHR_SINGLE_VALID)
    return EINVAL;
  code = pthread_mutex_lock( &(single->mutex) );
  if (code != 0)
    return code;
  if (single->counter != single->threshold) {
    /* Some pthreads are waiting in the single struct
       Do nothing */
    pthread_mutex_unlock( &(single->mutex) );
    return EBUSY; }
  /* Invalid the struct to avoid usage */
  single->valid = 0;
  code = pthread_mutex_unlock( &(single->mutex) );
  if (code != 0)
    return code;
  /* Destroy mutex & cond */
  code = pthread_mutex_destroy( &(single->mutex) );
  code2 = pthread_cond_destroy( &(single->cond) );
  return (code == 0 ? code : code2);
}
/* Wait in single section */
int pthr_single_wait(pthr_single_t *single)
{
  int cycle,code;
  /* Check validity */
  if (single->valid != PTHR_SINGLE_VALID)
    return EINVAL;
  code = pthread_mutex_lock( &(single->mutex) );
  if (code != 0)
    return code;
  if (single->first == 1) {
    /* We are the first one */
    single->first = 0;
    code = -1; }
  else {
    cycle = single->cycle;
    (single->counter)--;
    if (single->counter == 0) {
      /* We are the last one */
      single->cycle = !(single->cycle);
      /* Reinit the single struct */
      single->counter = single->threshold;
      single->first = 1;
      /* Wake the sleeping threads */
      code = pthread_cond_broadcast( &(single->cond) ); }
    else {
      /* Wait for broadcast wake up */
      /* Check cycle to avoid wake up from an oldest single struct */
      while (cycle == single->cycle) {
	code = pthread_cond_wait( &(single->cond), &(single->mutex) );
	if (code != 0)
	  break; } } }
  pthread_mutex_unlock( &(single->mutex) );
  return code;
}
