#ifndef PTHRSINGLE_H
#define PTHRSINGLE_H

/* Add the single section openmp like (with implicit wait)
  in pthreads.
  It's like a barrier. The first threads return from wait call
  with value -1 and can do some job, then he call again to wait the others.
  The other threads are blocked in the first call.

  Ex :
  if (pthr_single_wait( single ) == -1)
  {
    do_single_work();
    pthr_single_wait( single );
  }
 */
#include <pthread.h>

typedef struct pthrsingle {
  pthread_mutex_t mutex;
  pthread_cond_t cond; /* wait for the others in section */
  int valid; /* lightly check of valid usage */
  int threshold; /* number of threads in the section */
  int counter; /* Internal counter */
  int first; /* To known the first thread */
  int cycle; /* Enable more than one single section */
} pthr_single_t;
/* Valid number */
#define PTHR_SINGLE_VALID 0xabadcafe
/* Static Init */
#define PTHR_SINGLE_INITIALIZER(cnt) \
  {PTHREAD_MUTEX_INITIALIZER, PTHREAD_COND_INITIALIZER, \
      SINGLE_VALID, cnt, cnt, 1, 0}
/* Init the single structure with the numbers of threads involved */
extern int pthr_single_init(pthr_single_t * single, int count);
/* Destroy the single structure */
extern int pthr_single_destroy(pthr_single_t * single);
/* Wait in single section */
extern int pthr_single_wait(pthr_single_t *single);
#endif
