#include "pthrsinglelast.h"
#include <errno.h>
/* Init the singlelast structure with the numbers of threads involved */
int pthr_singlelast_init(pthr_singlelast_t *singlelast, int count) {
  int code;

  code = pthread_mutex_init( &(singlelast->mutex), NULL);
  if (code != 0)
    return code;
  code = pthread_cond_init( &(singlelast->cond), NULL);
  if (code != 0) {
    pthread_mutex_destroy( &(singlelast->mutex) );
    return code; }
  singlelast->valid = PTHR_SINGLELAST_VALID;
  singlelast->threshold = count;
  singlelast->counter = count;
  singlelast->last = 1;
  singlelast->cycle = 0;
  return 0; }
/* Destroy the single structure */
int pthr_singlelast_destroy(pthr_singlelast_t *singlelast) {
  int code,code2;
  if (singlelast->valid != PTHR_SINGLELAST_VALID)
    return EINVAL;
  code = pthread_mutex_lock( &(singlelast->mutex) );
  if (code != 0)
    return code;
  if (singlelast->counter != singlelast->threshold) {
    pthread_mutex_unlock( &(singlelast->mutex) );
    return EBUSY; }
  /* Invalid object */
  singlelast->valid = 0;
  code = pthread_mutex_unlock( &(singlelast->mutex) );
  if (code != 0)
    return code;
  code = pthread_mutex_destroy( &(singlelast->mutex) );
  code2 = pthread_cond_destroy( &(singlelast->cond) );
  return (code == 0 ? code : code2);}
/* Wait in single section */
int pthr_singlelast_wait(pthr_singlelast_t *singlelast) {
  int cycle,code;
  /* Check validity of object */
  if (singlelast->valid != PTHR_SINGLELAST_VALID)
    return EINVAL;
  code = pthread_mutex_lock( &(singlelast->mutex) );
  if (code != 0)
    return code;
  cycle = singlelast->cycle;
  (singlelast->counter)--;
  if (singlelast->counter <= 0) {
    /* The last thread */
    if (singlelast->last == 1) {
      /* On first call return */
      singlelast->last = 0;
      code = -1;
    } else {
      /* In second call wake up the others threads */
      singlelast->cycle = !(singlelast->cycle);
      singlelast->counter = singlelast->threshold;
      singlelast->last = 1;
      code = pthread_cond_broadcast( &(singlelast->cond) ); }
  } else {
    while (cycle == singlelast->cycle) {
      /* Wait for the wake up */
      code = pthread_cond_wait( &(singlelast->cond), &(singlelast->mutex) );
      if (code != 0)
	break; }}
  pthread_mutex_unlock( &(singlelast->mutex) );
  return code; }

