#ifndef _PTHRSINGLELAST_H
#define _PTHRSINGLELAST_H
#include <pthread.h>
/* The singlelast structure provide a single section where the last
   threads do the work. It can replace a barrier followed by
   a single with wait.
  Ex :
  if (pthr_singlelast_wait( single ) == -1)
  {
    do_single_work();
    pthr_singlelast_wait( single );
  }
 */
typedef struct singlelast {
  pthread_mutex_t mutex;
  pthread_cond_t cond; /* wait for the others in section */
  int valid; /* lightly check of valid usage */
  int threshold; /* number of threads in the section */
  int counter; /* Internal counter */
  int last; /* To known the last thread */
  int cycle; /* Enable more than one single section */
} pthr_singlelast_t;
/* Valid number */
#define PTHR_SINGLELAST_VALID 0xabadcafe
/* Static Init */
#define PTHR_SINGLELAST_INITIALIZER(cnt) \
  {PTHREAD_MUTEX_INITIALIZER, PTHREAD_COND_INITIALIZER, \
      PTHR_SINGLELAST_VALID, cnt, cnt, 1, 0}
/* Init the singlelast structure with the numbers of threads involved */
extern int pthr_singlelast_init(pthr_singlelast_t *singlelast, int count);
/* Destroy the single structure */
extern int pthr_singlelast_destroy(pthr_singlelast_t *singlelast);
/* Wait in single section */
extern int pthr_singlelast_wait(pthr_singlelast_t *singlelast);
#endif
