!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! -*- Mode: F90 -*- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! main.f90 --- 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

program hydro_main
  use hydro_commons
  use hydro_parameters
  use hydro_IO
  use hydro_mpi
  use hydro_principal
  use Ptim
  use omp_parameters
  use omp_domain_decomposition
  implicit none

  real(kind=prec_real)   :: dt, tps_start_iter, tps_stop_iter

  !--MPI--!
  ! Initialization of MPI environment
  call MPI_INIT_THREAD(MPI_THREAD_FUNNELED,level_mpi_provided,code)
  ! MPI timing
  call PTIM_start(LABEL="Hydro MPI-OpenMP")
  !--MPI--!

  ! Read run parameters
  call read_params

  ! Initialize MPI domains and communicator
  call init_mpi

  !$OMP PARALLEL
  ! Initialize OpenMP domains
  call init_omp

  if (rang==0) then
     !$OMP SINGLE
     print *
     print *,'*** HYDRO_Hybride MPI/OpenMP ***'
     print *,' Execution with ',nb_procs,' MPI processes (',dimsMPI(1),'x',dimsMPI(2),')'
     print *,' and ',nb_th,' OpenMP threads (',dims(1),'x',dims(2),') per MPI process'
     print *
     print *,'Starting time integration with nx = ',nxInit,' ny = ',nyInit
     print *
     !$OMP END SINGLE
  endif
  !$OMP SINGLE
  deallocate(dims)
  !$OMP END SINGLE


  ! Initialize hydro grid
  call init_hydro

  ! Main time loop
  do while (t < tend .and. nstep < nstepmax)

     ! Start timing iteration
     if (rang==0 .and. num_th==0) tps_start_iter = MPI_WTIME()

     ! Output results
     if( on_output .and. MOD(nstep,noutput)==0)then
!$OMP SINGLE
        call output
!$OMP END SINGLE
     end if

     ! Compute new time-step
     if(MOD(nstep,2)==0)then
        call cmpdt(dt)
        !$OMP SINGLE
        if(nstep==0)dt=dt/2.
        !$OMP END SINGLE
     endif

     ! Directional splitting
     if(MOD(nstep,2)==0)then
        call godunov(1,dt)
        call godunov(2,dt)
     else
        call godunov(2,dt)
        call godunov(1,dt)
     end if

     !$OMP SINGLE
     nstep=nstep+1
     t=t+dt
     !$OMP END SINGLE
     !--MPI--!
     if (rang==0 .and. num_th==0) then
        tps_stop_iter = MPI_WTIME()
        write(*,'("step=",I6," t=",1pe10.3," dt=",1pe10.3," (",0pf8.3," s.)")') &
             nstep,t,dt,tps_stop_iter-tps_start_iter
     endif
     !--MPI--!

  end do

  ! Final output
  !$OMP SINGLE
  if (on_output) call output
  !$OMP END SINGLE
  !$OMP END PARALLEL

  !--MPI--!
  ! Free MPI structures
  call clean_mpi
  ! MPI timing result
  call PTIM_stop(LABEL="Hydro MPI-OpenMP")
  ! Shutting down MPI environment
  call MPI_FINALIZE(code)
  !--MPI--!
 
end program hydro_main
