!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! -*- Mode: F90 -*- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! main.f90 --- 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

program hydro_main
  use hydro_commons
  use hydro_mpi
  use hydro_parameters
  use hydro_IO
  use hydro_principal
  use Ptim
  implicit none

  real(kind=prec_real)   :: dt, tps_start_iter, tps_stop_iter

  !--MPI--!
  ! Initialization of MPI environment
  required = MPI_THREAD_SERIALIZED
  call MPI_INIT_THREAD(required,provided,code)
  if(nthreads>1.and.provided<required)then
    print *,'Multithreading level too small'
  endif
  ! MPI timing
  call PTIM_start(LABEL="Hydro MPI2D")
  !--MPI--!

  ! Read run parameters
  call read_params

  ! Initialize MPI domains and communicator
  call init_mpi

  !$OMP PARALLEL
  ! Initialize hydro grid
  call init_hydro

  ! Main time loop
  do while (t < tend .and. nstep < nstepmax)

     ! Start timing iteration
     !$OMP SINGLE
     if (rang==0) tps_start_iter = MPI_WTIME()
     !$OMP END SINGLE

     ! Output results
     if( on_output .and. MOD(nstep,noutput)==0)then
        !$OMP SINGLE
        call output
        !$OMP END SINGLE
      end if

     ! Compute new time-step
     if(MOD(nstep,2)==0)then
        call cmpdt(dt)
        !$OMP SINGLE
        if(nstep==0)dt=dt/2.
        !$OMP END SINGLE
     endif

     ! Directional splitting
     if(MOD(nstep,2)==0)then
        call godunov(1,dt)
        call godunov(2,dt)
     else
        call godunov(2,dt)
        call godunov(1,dt)
     end if

     !$OMP SINGLE
     nstep=nstep+1
     t=t+dt
     !--MPI--!
     if (rang==0) then
        tps_stop_iter = MPI_WTIME()
        write(*,'("step=",I6," t=",1pe10.3," dt=",1pe10.3," (",0pf8.3," s.)")') &
             nstep,t,dt,tps_stop_iter-tps_start_iter
     endif
     !--MPI--!
     !$OMP END SINGLE

  end do

  ! Final output
  !$OMP SINGLE
  if (on_output) call output
  !$OMP END SINGLE
  !$OMP END PARALLEL

  !--MPI--!
  ! Free MPI structures
  call clean_mpi
  ! MPI timing result
  call PTIM_stop(LABEL="Hydro MPI2D")
  ! Shutting down MPI environment
  call MPI_FINALIZE(code)
  !--MPI--!

end program hydro_main
