!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! -*- Mode: F90 -*- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! main.f90 --- 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

program hydro_main
  use mpi_parameters
  use hydro_commons
  use hydro_parameters
  use hydro_IO
  use hydro_principal
  use Ptim
  implicit none

  real(kind=prec_real)   :: dt

  !--MPI--!
  ! Initialization of MPI environment
  call MPI_INIT(code)
  ! MPI timing
  call PTIM_start(LABEL="Hydro MPI1D")
  !--MPI--!

  ! Read run parameters
  call read_params

  ! Initialize hydro grid
  call init_hydro

  !--MPI--!
  if (rang==0) then
     call MPI_COMM_SIZE(MPI_COMM_WORLD,nb_procs,code)
     print *
     print *,' MPI Execution with ',nb_procs,' process'
     print *,' Starting time integration, nx = ',nxi,' ny = ',nyi  
     print *
  endif
  !--MPI--!

  ! Main time loop
  do while (t < tend .and. nstep < nstepmax)

     ! Output results
     if( on_output .and. MOD(nstep,noutput)==0)then
        call output
     end if

     ! Compute new time-step
     if(MOD(nstep,2)==0)then
        call cmpdt(dt)
        if(nstep==0)dt=dt/2.
     endif

     ! Directional splitting
     if(MOD(nstep,2)==0)then
        call godunov(1,dt)
        call godunov(2,dt)
     else
        call godunov(2,dt)
        call godunov(1,dt)
     end if

     nstep=nstep+1
     t=t+dt
     !--MPI--!
      if (rang==0) write(*,'("step=",I6," t=",1pe10.3," dt=",1pe10.3)')nstep,t,dt
      !--MPI--!

  end do

  ! Final output
  if (on_output) call output

  !--MPI--!
  ! MPI timing result
  call PTIM_stop(LABEL="Hydro MPI1D")
  ! Shutting down MPI environment
  call MPI_FINALIZE(code)
  !--MPI--!
 
end program hydro_main
