!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! -*- Mode: F90 -*- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! main.f90 --- 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

program hydro_main
  use hydro_commons
  use hydro_parameters
  use hydro_IO
  use hydro_principal
  use omp_parameters
!$ use OMP_LIB

  implicit none

  real(kind=prec_real)   :: dt, tps_elapsed
  integer(kind=prec_int) :: nbp_init, nbp_final, nbp_max, freq_p

  ! Initialize clock counter
  call system_clock(count_rate=freq_p, count_max=nbp_max)
  call system_clock(nbp_init)

  ! Read run parameters
  call read_params

  !$OMP PARALLEL 
  ! Initialize hydro grid
  call init_hydro

  !$OMP SINGLE
  print *
  print *,'HYDRO_OMP1D_NoSync - Execution with ',omp_get_num_threads(),' threads'
  print *,'Starting time integration with nx = ',nx,' ny = ',ny
  print *
  !$OMP END SINGLE

  ! Main time loop
  do while (t < tend .and. nstep < nstepmax)

     ! Output results
     if( on_output .and. MOD(nstep,noutput)==0)then
!$OMP SINGLE
        call output
!$OMP END SINGLE
     end if

     ! Compute new time-step
     if(MOD(nstep,2)==0)then
        call cmpdt(dt)
        !$OMP SINGLE
        if(nstep==0)dt=dt/2.
        !$OMP END SINGLE
     endif

     ! Directional splitting
     if(MOD(nstep,2)==0)then
        call godunov(1,dt)
        call godunov(2,dt)
     else
        call godunov(2,dt)
        call godunov(1,dt)
     end if

     !$OMP SINGLE
     nstep=nstep+1
     t=t+dt
     write(*,'("step=",I6," t=",1pe10.3," dt=",1pe10.3)')nstep,t,dt
     !$OMP END SINGLE

  end do

  ! Final output
  !$OMP SINGLE
  if (on_output) call output
  !$OMP END SINGLE
!    write(*,'("step=",I6," t=",1pe10.3," dt=",1pe10.3)')nstep,t,dt
  !$OMP END PARALLEL


  ! Timing
  call system_clock(nbp_final)
  if (nbp_final>nbp_init) then
     tps_elapsed=real(nbp_final-nbp_init)/real(freq_p)
  else
     tps_elapsed=real(nbp_final-nbp_init+nbp_max)/real(freq_p) 
  endif  

  print *
  print *,'Temps elapsed (s.) : ',tps_elapsed
  print *

end program hydro_main
