!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! -*- Mode: F90 -*- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! module_hydro_IO.f90 --- 
!!!!
!! subroutine read_params
!! subroutine output 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

module hydro_IO

contains

subroutine read_params
  use hydro_parameters
  implicit none

  ! Local variables
  integer(kind=prec_int) :: narg,iargc
  character(LEN=80) :: infile

  ! Namelists
  namelist/run/nstepmax,tend,noutput,on_output
  namelist/mesh/nx,ny,dx,boundary_left,boundary_right,boundary_down,boundary_up, &
                idimbloc,jdimbloc
  namelist/hydro/gamma,courant_factor,smallr,smallc,niter_riemann, &
                 iorder,scheme,slope_type

  narg = iargc()
  IF(narg .NE. 1)THEN
     write(*,*)'You should type: a.out input.nml'
     write(*,*)'File input.nml should contain a parameter namelist'
     STOP
  END IF
  CALL getarg(1,infile)
  open(1,file=infile)
  read(1,NML=run)
  read(1,NML=mesh)
  read(1,NML=hydro)
  close(1)
end subroutine read_params


subroutine output
  !$ use OMP_LIB
  use hydro_commons
  use hydro_parameters
  implicit none

  ! Local variables
  character(LEN=80) :: filename
  character(LEN=5)  :: char,charpe
  integer(kind=prec_int) :: nout
  integer(kind=prec_int) :: numIO, rang
  integer(kind=prec_int) :: nb_th, j_deb, j_fin, j_taille

  rang = 0
  nb_th = 1
!  !$ rang = omp_get_thread_num()
!  !$ nb_th = omp_get_num_threads()
  if (rang<mod(ny,nb_th)) then
     j_taille = ny/nb_th+1
     j_deb = rang*j_taille+1
     j_fin = (rang+1)*j_taille
  else
     j_taille=ny/nb_th
     j_deb = ny-(nb_th-rang)*j_taille+1
     j_fin = ny-(nb_th-rang-1)*j_taille
  endif

  numIO=10+rang
  nout=nstep/noutput
  call title(nout,char)
  call title(rang,charpe)
  filename='output_'//TRIM(char)//'.'//TRIM(charpe)
  open(numIO,file=filename,form='unformatted')
  rewind(numIO)
  if (rang == 0) print*,'Outputting array of size=',nx,j_taille,nvar
  write(numIO)real(t,kind=prec_output),real(gamma,kind=prec_output)
  write(numIO)nx,j_taille,nvar,nstep
!  write(numIO)real(uold(imin+2:imax-2,j_deb+2:j_fin-2,1:nvar),kind=prec_output)
  write(numIO)real(uold(imin+2:imax-2,jmin+2:jmax-2,1:nvar),kind=prec_output)
  close(numIO)

contains

subroutine title(n,nchar)
  use hydro_precision
  implicit none

  integer(kind=prec_int), intent(in) :: n
  character(LEN=5), intent(out) :: nchar
  character(LEN=1) :: nchar1
  character(LEN=2) :: nchar2
  character(LEN=3) :: nchar3
  character(LEN=4) :: nchar4
  character(LEN=5) :: nchar5

  if(n.ge.10000)then
     write(nchar5,'(i5)') n
     nchar = nchar5
  elseif(n.ge.1000)then
     write(nchar4,'(i4)') n
     nchar = '0'//nchar4
  elseif(n.ge.100)then
     write(nchar3,'(i3)') n
     nchar = '00'//nchar3
  elseif(n.ge.10)then
     write(nchar2,'(i2)') n
     nchar = '000'//nchar2
  else
     write(nchar1,'(i1)') n
     nchar = '0000'//nchar1
  endif
end subroutine title

end subroutine output

end module hydro_IO
